// Crimson Fields -- a game of tactical warfare
// Copyright (C) 2000-2003 Jens Granseuer
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

///////////////////////////////////////////////////////////////
// mapwindow.h
//
// Changes:
//   22-04-2001 - added FOG_ALPHA
///////////////////////////////////////////////////////////////

#ifndef _INCLUDE_MAPWINDOW_H
#define _INCLUDE_MAPWINDOW_H

#include "window.h"
#include "mapview.h"
#include "mapwidget.h"
#include "misc.h"

#define ANIM_SPEED_UNIT		150  // time it takes for a unit to move one hex (ms)
#define ANIM_SPEED_CURSOR	40   // speed when moveing cursor

#define DEFAULT_PANEL_HEIGHT	(20)

// the small info panel at the bottom of the map window
class Panel : public Window {
public:
  Panel( class MapWindow *mw, View *view );

  void Draw( void );
  void Update( MapObject *obj );
  virtual GUI_Status HandleEvent( const SDL_Event &event );
  virtual void VideoModeChange( void );

private:
  MapObject *obj;
  MapWindow *mapwin;
};



class MapWindow : public Window {
public:
  MapWindow( short x, short y, unsigned short w, unsigned short h,
             unsigned short flags, View *view );
  ~MapWindow( void ) { delete mview; }

  virtual GUI_Status HandleEvent( const SDL_Event &event );
  virtual void Draw( void );
  virtual void Close( void ) { Window::Close(); view->CloseWindow(panel); }
  virtual void DrawBack( short x, short y, unsigned short w, unsigned short h );
  virtual void VideoModeChange( void );

  MapView *GetMapView( void ) const { return mview; }
  Panel *GetPanel( void ) const { return panel; }

  void MoveHex( unsigned short img, Surface *surface, const Point &hex1,
                const Point &hex2, unsigned short speed );
  Point MoveCursor( Direction dir );
  void BoxAvoidHexes( Rect &rect, const Point &hex1, const Point &hex2 ) const;

  void DrawUnitInfo( const UnitType *type, Surface *dest, const Rect &rect );

private:
  MapView *mview;
  Panel *panel;
};


class TacticalWindow : public Window, public WidgetHook {
public:
  TacticalWindow( MapView *mapview, View *view );

  void Draw( void );
  GUI_Status WidgetActivated( Widget *widget, Window *win );

private:
  void CalcViewPort( Rect &vp ) const;

  MapView *mv;
  MapWidget *mw;
};

#endif	/* _INCLUDE_MAPWINDOW_H */

