// CoMET - The Crimson Fields Map Editing Tool
// Copyright (C) 2002-2003 Jens Granseuer
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

//////////////////////////////////////////////////////////////
// edwindow.h
//
// Changes:
///////////////////////////////////////////////////////////////

#ifndef _INCLUDE_ED_EDWINDOW_H
#define _INCLUDE_ED_EDWINDOW_H

#include "window.h"
#include "mapview.h"
#include "mission.h"
#include "gfxwidget.h"
#include "button.h"

#define ED_MODE_TERRAIN 1
#define ED_MODE_UNIT    2
#define ED_MODE_VIEW    3


class EdWindow : public Window, public WidgetHook {
public:
  EdWindow( const string &mapdir, View *view );
  ~EdWindow( void );

  MapView &GetMapView( void ) { return mv; }

  void Draw( void );
  GUI_Status HandleEvent( const SDL_Event &event );
  void Quit( void ) const;
  void MoveCursor( int key );
  void SetCursor( const Point &cursor );

  Mission *LoadMission( const char *file, bool filereq );
  GUI_Status SaveMission( const char *file, bool filereq );
  GUI_Status NewMission( const Point &size, const string &tset,
                         const string &uset );
  GUI_Status ExportMission( const char *file, bool filereq );
  string GetFileName( const char *filename, const char *suffix,
                      const string &dirname, int flag ) const;

private:
  GUI_Status WidgetActivated( Widget *widget, Window *win );
  void LeftMouseButton( const Point &hex );
  void ShowContextMenu( const Point &clicked );
  void PrintCursorPos( void );

  void SwapTiles( unsigned short t1, unsigned short t2 );

  static const short L_ID_UNITS = 0;
  static const short L_ID_TILES = 1;
  static const short S_ID_VERT  = 2;
  static const short S_ID_HORIZ = 3;

  Mission *mission;
  Rect panel;
  MapView mv;

  unsigned char mode;
  short selected;
  Point selected_hex;

  TileWidget *tile_wd;
  UnitWidget *unit_wd;
  SliderWidget *ud_wd;  // up/down
  SliderWidget *lr_wd;  // left/right

  string mapdir;
};

#endif  // _INCLUDE_ED_EDWINDOW_H

