#include <inklevel.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

void usage(void) {
  printf("ink -p<usb|parport> -n<portnumber>\n");
}

int main(int argc, char *argv[]) {
  struct ink_level *level = NULL;
  int result = 0;
  int port = 0;
  int portnumber = 0;
  char c;

  printf("ink v0.1 (c) 2003 Markus Heinz\n\n");

  if (argc != 3) {
    usage();
    return 1;
  }

  while ((c = getopt(argc, argv, "p:n:")) != -1) {
    switch (c) {
    case 'p':
      if (strcmp(optarg, "parport") == 0) {
	port = PARPORT;
      } else if (strcmp(optarg, "usb") == 0) {
	port = USB;
      } else {
	usage();
	return 1;
      }
      break;
    case 'n':
      if (optarg) {
	portnumber = atoi(optarg);
      } else {
	usage();
	return 1;
      }
      break;
    default:
      usage();
      return 1;
    }
  }

  level = (struct ink_level *) malloc(sizeof(struct ink_level));

  if (level == NULL) {
    printf("Not enough memory available.\n");
    return 1;
  }
  
  result = get_ink_level(port, portnumber, level);

  if (result == ERROR) {
    printf("Could not get ink level.\n");
    free(level);
    return 1;
  }

  switch (level->type) {
  case NO_INK_LEVEL_FOUND:
    printf("No ink level found\n");
    break;
  case ONE_COLOR_FOUND:
    printf("Color: %d%%\n", level->color);
    break;
  case ONE_BLACK_COLOR_FOUND:
    printf("Black: %d%%\n", level->black);
    break;
  case TWO_COLORS_FOUND:
    printf("Black: %d%%\n", level->black);
    printf("Color: %d%%\n", level->color);
    break;
  case TWO_PHOTO_COLORS_FOUND:
    printf("Photo: %d%%\n", level->photo);
    printf("Color: %d%%\n", level->color);
    break;
  case THREE_COLORS_FOUND:
    printf("Cyan: %d%%\n", level->cyan);
    printf("Magenta: %d%%\n", level->magenta);
    printf("Yellow: %d%%\n", level->yellow);
    break;
  case FOUR_COLORS_FOUND:
    printf("Black %d%%\n", level->black);
    printf("Cyan: %d%%\n", level->cyan);
    printf("Magenta: %d%%\n", level->magenta);
    printf("Yellow: %d%%\n", level->yellow);
    break;
  case SIX_COLORS_FOUND:
    printf("Black: %d%%\n", level->black);
    printf("Photocyan: %d%%\n", level->photocyan);
    printf("Photomagenta: %d%%\n", level->photomagenta);
    printf("Cyan: %d%%\n", level->cyan);
    printf("Magenta: %d%%\n", level->magenta);
    printf("Yellow: %d%%\n", level->yellow);
    break;
  }

  free(level);

  return 0;
}
